library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity effector is
port (
	clk : in std_logic;
	audio_clk : in std_logic;
	reset_n : in std_logic;

	-- for temporal testing... we will include this as bus commande from CPU!
	Ldistortion_enable : in std_logic;
	Lclip : in std_logic_vector(15 downto 0);	

	Rdistortion_enable : in std_logic;
	Rclip : in std_logic_vector(15 downto 0);	

    Lvibrato_enable: in std_logic;
    Lvibrato_amp: in std_logic_vector(2 downto 0);
	Lvibrato_freq: in std_logic_vector(3 downto 0);

    Rvibrato_enable: in std_logic;
    Rvibrato_amp: in std_logic_vector(2 downto 0);
	Rvibrato_freq: in std_logic_vector(3 downto 0);
	
	Lchorus_enable: in std_logic;
	Lchorus_amp: in std_logic_vector(2 downto 0);
	Lchorus_freq: in std_logic_vector(3 downto 0);
	Lchorus_mix_delay: in std_logic_vector(3 downto 0);

	Rchorus_enable: in std_logic;
	Rchorus_amp: in std_logic_vector(2 downto 0);
	Rchorus_freq: in std_logic_vector(3 downto 0);
	Rchorus_mix_delay: in std_logic_vector(3 downto 0);
	
	Lvol_valid: in std_logic;
	Lvol_data: in std_logic_vector(6 downto 0);

	Rvol_valid: in std_logic;
	Rvol_data: in std_logic_vector(6 downto 0);

    -- Audio interface signals
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic;  --    Audio CODEC Bit-Stream Clock
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;   --    Audio CODEC DAC Data
	I2C_SCLK	 : out std_logic;
	I2C_SDAT	 : inout std_logic
);
end effector;


architecture rtl of effector is

component de2_wm8731_audio_in is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    data_out : out std_logic_vector(15 downto 0);
    audio_req : out std_logic;

    -- Audio interface signals
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  component;

signal data_from_ADC_to_LR_in : std_logic_vector(15 downto 0);
signal request_from_ADC_to_LR_in : std_logic;


component LRRequestBuffer_in is
port (
    clk : in std_logic;    
	lrck : in std_logic;
    audio_req : in std_logic;
    data_in : in std_logic_vector (15 downto 0);	

	data_left : out std_logic;
	data_right : out std_logic;
    dataL_out : out std_logic_vector(15 downto 0);
    dataR_out : out std_logic_vector(15 downto 0)
    
  );
end  component;

signal dataL_from_LR_in_to_Lchorus : std_logic_vector(15 downto 0);
signal dataR_from_LR_in_to_Rchorus : std_logic_vector (15 downto 0);

signal data_req_from_LR_in_to_Lchorus : std_logic;
signal data_req_from_LR_in_to_Rchorus : std_logic;

component chorus is
port (
       --In
       clk: in std_logic;      --Should be 50 MHz
	   reset_n: in std_logic;
       data_enable: in std_logic;
       chorus_enable: in std_logic;
	   delay_amp: in std_logic_vector(2 downto 0);
	   delay_freq: in std_logic_vector(3 downto 0);
	   mix_delay: in std_logic_vector(3 downto 0);
       data_in: in std_logic_vector(15 downto 0);

       --Out
	   data_ready: out std_logic;
       data_out: out std_logic_vector(15 downto 0)
       );
end component;

signal data_ready_from_Lchorus_to_Lvibrato : std_logic;
signal data_ready_from_Rchorus_to_Rvibrato : std_logic;

signal data_from_Lchorus_to_Lvibrato : std_logic_vector(15 downto 0);
signal data_from_Rchorus_to_Rvibrato : std_logic_vector(15 downto 0);

component vibrato is
port (
       --In
       clk: in std_logic;      --Should be 50 MHz
	   reset_n: in std_logic;
       data_enable: in std_logic;
       vibrato_enable: in std_logic;
	   delay_amp: in std_logic_vector(2 downto 0);
	   delay_freq: in std_logic_vector(3 downto 0);
	   data_in: in std_logic_vector(15 downto 0);

       --Out
	   data_ready: out std_logic;
       data_out: out std_logic_vector(15 downto 0)
       );
end component;

signal data_ready_from_Lvibrato_to_Ldistortion : std_logic;
signal data_ready_from_Rvibrato_to_Rdistortion : std_logic;

signal data_from_Lvibrato_to_Ldistortion : std_logic_vector(15 downto 0);
signal data_from_Rvibrato_to_Rdistortion : std_logic_vector(15 downto 0);

component distortion is
port (
	clk : in std_logic;
	reset_n : in std_logic;
	data_enable : in std_logic;
	distortion_enable : in std_logic;
	clip : in std_logic_vector(15 downto 0);	
	data_in : in std_logic_vector(15 downto 0);

	data_out : out std_logic_vector(15 downto 0);
	data_ready : out std_logic

);
end component;


signal data_ready_from_Ldistortion_to_LR_out : std_logic;
signal data_ready_from_Rdistortion_to_LR_out : std_logic;

signal data_from_Ldistortion_to_LR_out: std_logic_vector(15 downto 0);
signal data_from_Rdistortion_to_LR_out: std_logic_vector(15 downto 0);


component LRRequestBuffer_out is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
	lrck : in std_logic;
	data_left : in std_logic;
	data_right : in std_logic;
    dataL_in : in std_logic_vector(15 downto 0);
    dataR_in : in std_logic_vector(15 downto 0);
    audio_req : in std_logic;

    data_out : out std_logic_vector (15 downto 0)	
    
  );
end  component;

signal request_from_DAC_to_LR_out : std_logic;
signal data_from_LR_out_to_DAC : std_logic_vector(15 downto 0);


component de2_wm8731_audio_out is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    test_mode : in std_logic;       --    Audio CODEC controller test mode
    data : in std_logic_vector(15 downto 0);	
	audio_req : out std_logic;
  
    -- Audio interface signals
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic   --    Audio CODEC DAC Data
  );
end component;

signal internal_lrck_in, internal_lrck_out : std_logic;

component de2_i2c_av_config
port (
 iCLK		: in std_logic;
 iRST_N		: in std_logic;
 I2C_SCLK	: out std_logic;
 I2C_SDAT	: inout std_logic;
 left_valid	: in std_logic;
 left_data	: in std_logic_vector(6 downto 0);
 right_valid: in std_logic;
 right_data	: in std_logic_vector(6 downto 0)
);
end component;


begin

	AUD_ADCLRCK <= internal_lrck_in;
	AUD_DACLRCK <= internal_lrck_out;

	AUDIO_IN: de2_wm8731_audio_in port map(
	    clk => audio_clk,
	    reset_n => reset_n,
	    data_out => data_from_ADC_to_LR_in,
	    audio_req => request_from_ADC_to_LR_in,

	    -- Audio interface signals
		AUD_ADCLRCK  => internal_lrck_in,
	    AUD_ADCDAT   => AUD_ADCDAT,   --    Audio CODEC ADC Data
	    AUD_BCLK     => AUD_BCLK  --    Audio CODEC Bit-Stream Clock
	);


	BUFFER_IN: LRRequestBuffer_in port map(
	    clk => clk,    
		lrck => internal_lrck_in,
	    audio_req => request_from_ADC_to_LR_in,
	    data_in => 	data_from_ADC_to_LR_in,

		data_left => data_req_from_LR_in_to_Lchorus,
		data_right => data_req_from_LR_in_to_Rchorus,
	    dataL_out => dataL_from_LR_in_to_Lchorus,
	    dataR_out => dataR_from_LR_in_to_Rchorus 
  	);



 LCHO:chorus port map(
       --In
       clk => clk,     --Should be 50 MHz
	   reset_n => reset_n,
       data_enable => data_req_from_LR_in_to_Lchorus,
       chorus_enable => Lchorus_enable,
	   delay_amp => Lchorus_amp,
	   delay_freq => Lchorus_freq,
	   mix_delay => Lchorus_mix_delay,

       data_in => dataL_from_LR_in_to_Lchorus,
       --Out
	   data_ready => data_ready_from_Lchorus_to_Lvibrato,
       data_out => data_from_Lchorus_to_Lvibrato
 	);

 RCHO:chorus port map(
       --In
       clk => clk,     --Should be 50 MHz
	   reset_n => reset_n,
       data_enable => data_req_from_LR_in_to_Rchorus,
       chorus_enable => Rchorus_enable,
	   delay_amp => Rchorus_amp,
	   delay_freq => Rchorus_freq,
	   mix_delay => Rchorus_mix_delay,
       data_in => dataR_from_LR_in_to_Rchorus,
       --Out
	   data_ready => data_ready_from_Rchorus_to_Rvibrato,
       data_out => data_from_Rchorus_to_Rvibrato
 	);



	LVIB: vibrato port map(
       	--In
       	clk => clk,      --Should be 50 MHz
	   	reset_n => reset_n,
       	data_enable => data_ready_from_Lchorus_to_Lvibrato,
       	vibrato_enable => Lvibrato_enable,
		delay_amp => Lvibrato_amp,
	    delay_freq => Lvibrato_freq,
       	data_in => data_from_Lchorus_to_Lvibrato,

       	--Out
	   	data_ready => data_ready_from_Lvibrato_to_Ldistortion,
       	data_out => data_from_Lvibrato_to_Ldistortion
    );


	RVIB: vibrato port map(
	    --In
	    clk => clk,      --Should be 50 MHz
		reset_n => reset_n,
	    data_enable => data_ready_from_Rchorus_to_Rvibrato,
	    vibrato_enable => Rvibrato_enable,
		delay_amp => Rvibrato_amp,
	    delay_freq => Rvibrato_freq,
		data_in => data_from_Rchorus_to_Rvibrato,

	       --Out
		data_ready => data_ready_from_Rvibrato_to_Rdistortion,
		data_out => data_from_Rvibrato_to_Rdistortion
    );

	LDIS: distortion port map (
		clk => clk,
		reset_n => reset_n,
		data_enable => data_ready_from_Lvibrato_to_Ldistortion,
		distortion_enable => Ldistortion_enable,
		clip => Lclip,
		data_in => data_from_Lvibrato_to_Ldistortion,

		data_out => data_from_Ldistortion_to_LR_out,
		data_ready => data_ready_from_Ldistortion_to_LR_out
	);

	RDIS: distortion port map (
		clk => clk,
		reset_n => reset_n,
		data_enable => data_ready_from_Rvibrato_to_Rdistortion,
		distortion_enable => Rdistortion_enable,
		clip => Rclip,
		data_in => data_from_Rvibrato_to_Rdistortion,

		data_out => data_from_Rdistortion_to_LR_out,
		data_ready => data_ready_from_Rdistortion_to_LR_out
	);

	BUFFER_OUT: LRRequestBuffer_out port map(
	    clk => clk,    
		lrck => internal_lrck_out,
	 	data_left => data_ready_from_Ldistortion_to_LR_out,
		data_right => data_ready_from_Rdistortion_to_LR_out,
	    dataL_in => data_from_Ldistortion_to_LR_out,
	    dataR_in => data_from_Rdistortion_to_LR_out,
	    audio_req => request_from_DAC_to_LR_out,

	    data_out => data_from_LR_out_to_DAC    
	 );


	AUDIO_OUT: de2_wm8731_audio_out port map(
	    clk => audio_clk,
	    reset_n => reset_n,
	    test_mode => '0',
	    data => data_from_LR_out_to_DAC,
		audio_req => request_from_DAC_to_LR_out,
	  
	    -- Audio interface signals
	    AUD_DACLRCK  => internal_lrck_out,
	    AUD_DACDAT   => AUD_DACDAT
  	);


  i2c : de2_i2c_av_config port map (
    iCLK     => clk,
    iRST_n   => reset_n,
    I2C_SCLK => I2C_SCLK,
    I2C_SDAT => I2C_SDAT,
	left_valid => Lvol_valid,
	right_valid => Rvol_valid,
	left_data => Lvol_data,
	right_data => Rvol_data
  );

end rtl;
