/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */
//13 17

//LCD 80A00 to 80A0F
//effector: 80800 809FF
//
#include <stdio.h>
#include <alt_types.h>
#include "alt_up_ps2_port.h"
#include "LCD.h"
//#include "ps2_keyboard.h"

#define EFFECTOR_BASE 0x80800

#define LVOL 0
#define RVOL 2

#define LDIS_EN 4
#define LDIS_CLIP 6
#define RDIS_EN 8
#define RDIS_CLIP 10

#define LVIB_EN 12
#define LVIB_AMP 14
#define LVIB_FREQ 16

#define RVIB_EN 18
#define RVIB_AMP 20
#define RVIB_FREQ 22

#define LCHO_EN 24
#define LCHO_AMP 26
#define LCHO_FREQ 28
#define LCHO_MIX 30

#define RCHO_EN 32
#define RCHO_AMP 34
#define RCHO_FREQ 36
#define RCHO_MIX 38

#define KEY_NORMAL 0
#define KEY_EXTEND 1
#define KEY_RELEASE 2
#define KEY_EXTEND_RELEASE 3

#define KEY_RELEASED 0
#define KEY_PRESSED 1

// D: distortion enable
// V: vibrato enable
// C: chorus enable

// ARROW UP : volume up
// ARROW DOWN : volume down

// 1 : select chorus
// 2 : select vibrato
// 3 : select distortion

// Insert : conf 1 up (chorus, vibrato : AMP, distortion : CLIP)
// Delete : conf 1 down (chorus, vibrato : AMP, distortion : CLIP)
// Home : conf 2 up (chorus, vibrato : FREQ, distortion : NONE)
// End : conf 2 down (chorus, vibrato : FREQ, distortion : NONE)
// Page Up : conf 3 up (chorus : MIX, distortion, vibrato : NONE)
// Page Down : conf 3 down (chorus : MIX, distortion, vibrato : NONE)

//#define IO_EFFECTOR (x,y) { volatile short int *temp = 0x80800; *(temp+x) = y;}

int key_state;
int key_pressed;
// key_board = 0x80A10;
#define KEY_BASE 0x80A10

int main()
{
  short int *EFFECTOR = (short int *)0x80800;
  char display1[16];
  char display2[16];
  int disp_changed = 0;
  printf("Please wait three seconds to initialize keyboard\n");
  int code;
   LCD_Init();
//   LCD_Show_Text("YO WORK!");
   
   key_state = KEY_NORMAL;
   key_pressed = KEY_RELEASED;
   
   IOWR_16DIRECT(EFFECTOR+LVOL,0,121);    //left volume = 0  (0 is in the middle of the volume range)
   IOWR_16DIRECT(EFFECTOR+RVOL,0,121);    //right volume = 0  (0 is in the middle of the volume range)
   
   IOWR_16DIRECT(EFFECTOR+LDIS_EN,0,0); //left dist enable = 0 (left distortion off)
   IOWR_16DIRECT(EFFECTOR+LDIS_CLIP,0,(1<<8)); //left dist clipping value = (2^8)
   
   IOWR_16DIRECT(EFFECTOR+RDIS_EN,0,0); //right dist same as left dist
   IOWR_16DIRECT(EFFECTOR+RDIS_CLIP,0,(1<<8));
   
   IOWR_16DIRECT(EFFECTOR+LVIB_EN,0,0); //left vibrato enable = 0 (left vibrato off)
   IOWR_16DIRECT(EFFECTOR+LVIB_AMP,0,4);
   IOWR_16DIRECT(EFFECTOR+LVIB_FREQ,0,0);

   IOWR_16DIRECT(EFFECTOR+RVIB_EN,0,0); //right vibrato same as left vibrato
   IOWR_16DIRECT(EFFECTOR+RVIB_AMP,0,4);
   IOWR_16DIRECT(EFFECTOR+RVIB_FREQ,0,0);

   IOWR_16DIRECT(EFFECTOR+LCHO_EN,0,0); //left chorus enable = 0 (left chorus off)
   IOWR_16DIRECT(EFFECTOR+LCHO_AMP,0,4);
   IOWR_16DIRECT(EFFECTOR+LCHO_FREQ,0,0);
   IOWR_16DIRECT(EFFECTOR+LCHO_MIX,0,8);

   IOWR_16DIRECT(EFFECTOR+RCHO_EN,0,0); //right chorus same as left chorus
   IOWR_16DIRECT(EFFECTOR+RCHO_AMP,0,4);
   IOWR_16DIRECT(EFFECTOR+RCHO_FREQ,0,0);
   IOWR_16DIRECT(EFFECTOR+RCHO_MIX,0,8);
   
   
   char dist_on = 0;   
   char vibr_on = 0;
   char chor_on = 0; 
   unsigned char volume = 121;
   char selected = 0;
   unsigned short dis_clip, chor_amp, chor_freq, chor_mix, vib_amp, vib_freq;
   dis_clip = 1<<8;
   chor_amp = vib_amp = 4;
   chor_freq = vib_freq = 0;
   chor_mix = 8;
   LCD_Show_Text("Guitar Effect");
   while(1) {
        while(!IORD_8DIRECT(KEY_BASE, 0));
        code = IORD_8DIRECT(KEY_BASE, 4);
//        printf(" %x pressed!",code);
        
        switch(code) {
        case 0xf0:
            switch(key_state) {
                case KEY_NORMAL:
                    key_state = KEY_RELEASE;
                    break;
                case KEY_EXTEND:
                    key_state = KEY_EXTEND_RELEASE;
                    break;
                case KEY_RELEASE:
                case KEY_EXTEND_RELEASE:
                    break;
            }
            break;
        case 0xe0:        
            switch(key_state) {
                case KEY_NORMAL:
                    key_state = KEY_EXTEND;
                    break;
                case KEY_EXTEND:
                case KEY_RELEASE:
                case KEY_EXTEND_RELEASE:
                    break;
             }
             break;
        case 0x23: //D
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
                // do something with pressing D
                if(dist_on) {
                    sprintf(display1,"Dist off");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LDIS_EN, 0, 0); //disabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RDIS_EN, 0, 0); //disabling right distortion                
                    dist_on = 0;
                }
                else {
                    sprintf(display1,"Dist on");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LDIS_EN, 0, 1); //enabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RDIS_EN, 0, 1); //enabling right distortion
                    dist_on = 1;
                }
                key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x21: //C
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
                if(chor_on) {
                    sprintf(display1, "Chorus off");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_EN, 0, 0); //disabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RCHO_EN, 0, 0); //disabling right distortion                
                    chor_on = 0;
                }
                else {
                    sprintf(display1,"Chorus on");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_EN, 0, 1); //enabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RCHO_EN, 0, 1); //enabling right distortion
                    chor_on = 1;
                }
                key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x2A: //V
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
                if(vibr_on) {
                    sprintf(display1,"Vibrato off");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_EN, 0, 0); //disabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RVIB_EN, 0, 0); //disabling right distortion                
                    vibr_on = 0;
                }
                else {
                    sprintf(display1,"Vibrato on");
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_EN, 0, 1); //enabling left distortion
                    IOWR_16DIRECT(EFFECTOR+RVIB_EN, 0, 1); //enabling right distortion
                    vibr_on = 1;
                }
                key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x16: // 1
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
                sprintf(display1,"Chrous selected");
                disp_changed = 1;
                selected = 0;
                key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x1E: // 2
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
                sprintf(display1,"Vibrato selected");
                disp_changed = 1;
                selected = 1;
                key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x26: // 3
            if(key_state == KEY_NORMAL && key_pressed == KEY_RELEASED) {
               sprintf(display1,"Dist selected");
               disp_changed = 1;
               selected = 2;
               key_pressed = KEY_PRESSED;   
            }
            if(key_state == KEY_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            } 
            key_state = KEY_NORMAL;
            break;
        case 0x75: // with e0 - UP Arrow
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                if(volume != 127) volume++;
                sprintf(display1,"vol = %d",volume-48);
                disp_changed = 1;
                IOWR_16DIRECT(EFFECTOR+LVOL,0,volume);
                IOWR_16DIRECT(EFFECTOR+RVOL,0,volume);
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x72: // with e0 - DOWN Arrow
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                if(volume != 48) volume--;
                sprintf(display1,"vol = %d",volume-48);
                disp_changed = 1;
                IOWR_16DIRECT(EFFECTOR+LVOL,0,volume);
                IOWR_16DIRECT(EFFECTOR+RVOL,0,volume);
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x70: // with e0 - Insert
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0: // chorus
                    if(chor_amp != 4) chor_amp++;
                    sprintf(display1,"C amp = %d",chor_amp);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_AMP,0,chor_amp);
                    IOWR_16DIRECT(EFFECTOR+RCHO_AMP,0,chor_amp);
                    break;
                case 1: 
                    if(vib_amp != 4) vib_amp++;// vibrato
                    sprintf(display1,"V amp = %d",vib_amp);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_AMP,0,vib_amp);
                    IOWR_16DIRECT(EFFECTOR+RVIB_AMP,0,vib_amp);
                    break;
                case 2: 
                    if(dis_clip != 32768) dis_clip <<= 1;// distortion
                    sprintf(display1,"D clip = %d",dis_clip);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LDIS_CLIP,0,dis_clip);
                    IOWR_16DIRECT(EFFECTOR+RDIS_CLIP,0,dis_clip);
                    break;
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x71: //with e0 - Delete
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0: // chorus
                    if(chor_amp != 0) chor_amp--;
                    sprintf(display1,"C amp = %d",chor_amp);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_AMP,0,chor_amp);
                    IOWR_16DIRECT(EFFECTOR+RCHO_AMP,0,chor_amp);
                    break;
                case 1: 
                    if(vib_amp != 0) vib_amp--;// vibrato
                    sprintf(display1,"V amp = %d",vib_amp);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_AMP,0,vib_amp);
                    IOWR_16DIRECT(EFFECTOR+RVIB_AMP,0,vib_amp);
                    break;
                case 2: 
                    if(dis_clip != 1) dis_clip >>= 1;// distortion
                    sprintf(display1,"D clip = %d",dis_clip);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LDIS_CLIP,0,dis_clip);
                    IOWR_16DIRECT(EFFECTOR+RDIS_CLIP,0,dis_clip);
                    break;
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x6C: //with e0 - Home
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0: // chorus
                    if(chor_freq != 15) chor_freq++;
                    sprintf(display1,"C freq = %d",chor_freq);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_FREQ,0,chor_freq);
                    IOWR_16DIRECT(EFFECTOR+RCHO_FREQ,0,chor_freq);
                    break;
                case 1: 
                    if(vib_freq != 15) vib_freq++;// vibrato
                    sprintf(display1,"V freq = %d",vib_freq);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_FREQ,0,vib_freq);
                    IOWR_16DIRECT(EFFECTOR+RVIB_FREQ,0,vib_freq);
                    break;
                case 2: 
                    break; 
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0X69: //with e0 - End
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0: // chorus
                    if(chor_freq != 0) chor_freq--;
                    sprintf(display1,"C freq = %d",chor_freq);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_FREQ,0,chor_freq);
                    IOWR_16DIRECT(EFFECTOR+RCHO_FREQ,0,chor_freq);
                    break;
                case 1: 
                    if(vib_freq != 0) vib_freq--;// vibrato
                    sprintf(display1,"V freq = %d",vib_freq);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LVIB_FREQ,0,vib_freq);
                    IOWR_16DIRECT(EFFECTOR+RVIB_FREQ,0,vib_freq);
                    break;
                case 2: 
                    break; 
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x7D: //with e0 - Page Up
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0:
                    if(chor_mix != 15) chor_mix++;
                    sprintf(display1,"C mix = %d",chor_mix);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_MIX,0,chor_mix);
                    IOWR_16DIRECT(EFFECTOR+RCHO_MIX,0,chor_mix);
                    break;
                case 1:
                case 2: 
                    break;
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        case 0x7A: //with e0 - Page Down
            if(key_state == KEY_EXTEND && key_pressed == KEY_RELEASED) {
                switch (selected)
                {
                case 0:
                    if(chor_mix != 0) chor_mix--;
                    sprintf(display1,"C mix = %d",chor_mix);
                    disp_changed = 1;
                    IOWR_16DIRECT(EFFECTOR+LCHO_MIX,0,chor_mix);
                    IOWR_16DIRECT(EFFECTOR+RCHO_MIX,0,chor_mix);
                    break;
                case 1:
                case 2: 
                    break;
                }
                key_pressed = KEY_PRESSED;
            }
            if(key_state == KEY_EXTEND_RELEASE && key_pressed == KEY_PRESSED) {
                key_pressed = KEY_RELEASED;
            }
            key_state = KEY_NORMAL;
            break;
        default:
            key_state = KEY_NORMAL;
            break;
        }
        
        if(disp_changed == 1) {
            LCD_Init();
            LCD_Show_Text(display1);
            LCD_Line2();
            sprintf(display2,"    %c%c%c %c %d", (dist_on)?'D':' ',(vibr_on)?'V':' ', (chor_on)?'C':' ',
                        (selected == 0)?'C':(selected ==1)?'V':'D',volume-48);
            LCD_Show_Text(display2); 
            disp_changed = 0;
        }
   }

  return 0;
}




