----------------------------------------------------------------
-- chorus module
--
-- Made by Navarun Jagatpal, April 10, 2007
-- Modifyed by Young Jin Yoon, April 12, 2007
-- Will be debugged by everybody....yeah~!
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity chorus is
port (
       --In
       clk: in std_logic;      --Should be 50 MHz
	   reset_n: in std_logic;
       data_enable: in std_logic;
       chorus_enable: in std_logic;
	   delay_amp: in std_logic_vector(2 downto 0);
	   delay_freq: in std_logic_vector(3 downto 0);
	   mix_delay: in std_logic_vector(3 downto 0);
       data_in: in std_logic_vector(15 downto 0);

       --Out
	   data_ready: out std_logic;
       data_out: out std_logic_vector(15 downto 0)
       );
end chorus;

architecture rtl of chorus is

	signal delta_delay: std_logic_vector(2 downto 0);
	signal freq: std_logic_vector(3 downto 0):= "0000";
	
	signal freq_cnt: std_logic_vector(3 downto 0):= "0000";
		
	signal delay: std_logic_vector(11 downto 0);

	signal data_internal : std_logic_vector (15 downto 0);
	
	signal buffer_in: std_logic_vector(11 downto 0);
	signal buffer_out: std_logic_vector(11 downto 0);

	signal ram_re: std_logic;
	signal ram_we: std_logic;
	signal ram_addr: std_logic_vector(10 downto 0);
	signal ram_data_out: std_logic_vector(15 downto 0);		
	signal ram_data_in: std_logic_vector(15 downto 0);		

	signal state: std_logic_vector(3 downto 0) := X"0";
	
	signal sine_data: std_logic_vector(11 downto 0);
	signal sine_addr: std_logic_vector(12 downto 0):= "0000000000000";
	signal sine_request: std_logic;
	
	signal sine_amplitude: std_logic_vector(11 downto 0);
	
	signal mix_ram_data_out : std_logic_vector(15 downto 0); 
	signal mix_data_internal : std_logic_vector(15 downto 0);
	signal mix_delay_internal : std_logic_vector (3 downto 0);
    component RAM_1633_16 is
port (
               --In:
               clk : in std_logic;
               we   : in std_logic; -- Write-enable
               re   : in std_logic; -- Read-enable
               addr : in std_logic_vector(10 downto 0); --Address: 11 bits. This allows 0 to 2047, but the only valid

			   --inout:                                                                                                --addresses range from 0 to 1632.
               data_out : out std_logic_vector(15 downto 0); --Data In: 16 bits
			   data_in	: in std_logic_vector(15 downto 0)
       );
	end component;

	component sine_table is
	port (
		clk : in std_logic;
		reset_n : in std_logic;
		data_request : in std_logic;
		amplitude : in std_logic_vector (2 downto 0);
		addr : in std_logic_vector (12 downto 0);
		
		data_out : out std_logic_vector(11 downto 0)
			
	);
	end component;


begin

    R1: RAM_1633_16 port map(
    	--In:
		clk => clk,
		we  => ram_we, -- Write-enable
		re  => ram_re, -- Read-enable
		addr => ram_addr, --Address: 11 bits. This allows 0 to 2047, but the only valid
		
		--inout:                                                                                                --addresses range from 0 to 1632.
		data_out  => ram_data_out, --Data In: 16 bits
		data_in => ram_data_in
       );

	S1: sine_table port map (
		clk => clk,
		reset_n => reset_n,
		data_request => sine_request,
		amplitude => delta_delay,
		addr => sine_addr,
		data_out => sine_data	
	);

	-- for giving delta value of delay
	process(clk,reset_n)
	begin
		if(reset_n = '0') then
			delta_delay <= "100";
		elsif(clk'event and clk='1') then
			case delay_amp is
			when "000" =>
				sine_amplitude <= "000000011111";
			when "001" =>
				sine_amplitude <= "000000111111";				
			when "010" =>
				sine_amplitude <= "000001111111";	 			
			when "011" =>
				sine_amplitude <= "000011111111";
			when "100" =>
				sine_amplitude <= "000111111111";
			when others =>
				sine_amplitude <= (others => 'X');
			end case;
	    	delta_delay <= delay_amp;
			mix_delay_internal <= mix_delay;
			freq <= delay_freq;
		end if;
	end process;
	

	process(clk,reset_n)
    begin
		if(reset_n = '0') then
			sine_addr <= "0000000000000";
			buffer_in <= "000000000000";
			data_ready <= '0';
			freq_cnt <= "0000";
		elsif(clk 'event and clk = '1') then
	       	if(chorus_enable = '1') then
				case state is
					when X"0" => -- get the next buffer in
						data_ready <= '0';
						if(data_enable = '1') then
		                	if(buffer_in = "011001100000") then  --if buffer_in = 1632
		                    	buffer_in <= "000000000000";
							else
								buffer_in <= buffer_in + 1;
		                    end if;
							data_internal <= data_in;
							state <= state + 1; 
						end if;
					when X"1" => -- save the current value
						ram_we <= '1';
						ram_re <= '0';
						ram_addr <= buffer_in(10 downto 0);
						ram_data_in <= data_internal; 
						state <= state + 1;
					when X"2" => -- set the sine_table to get the delay
						ram_we <= '0';
						if(sine_addr = "0010111011011") then
							sine_addr <= "0000000000000";
							freq_cnt <= "0000";
						elsif(freq_cnt = freq) then
							sine_addr <= sine_addr + 1;
							freq_cnt <= "0000";
						else
							freq_cnt <= freq_cnt + 1;
						end if;
						sine_request <= '1';
						state <= state + 1;
					when X"3" => -- get the delay value and apply (in + (not out+1))
						sine_request <= '0';
						delay <= sine_data + (1633 - sine_amplitude);
						--buffer_out <= (buffer_in + ((not (delay(11 downto 0)))+1));
						buffer_out <= (buffer_in + ((not (delay(11 downto 0)))+1));
						state <= state + 1;
					when X"4" => -- if in-out is negative, add buffer size
						if(buffer_out(11) = '1') then
							buffer_out <= buffer_out + 1633;
						end if;
						state <= state + 1;
					when X"5" => -- read the value from RAM
						ram_we <= '0';
						ram_re <= '1';
						ram_addr <= buffer_out(10 downto 0);
						state <= state + 1;
					when X"6" => -- apply it into out and send rising edge of data_ready
						ram_re <= '0';
						case mix_delay_internal is
						when X"0" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "11111111" & ram_data_out(15 downto 8);
							else
								mix_ram_data_out <= "00000000" & ram_data_out(15 downto 8);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3)) +
													("1111" & data_internal(15 downto 4)) +
													("11111" & data_internal(15 downto 5)) +
													("111111" & data_internal(15 downto 6)) +
													("1111111" & data_internal(15 downto 7)) +
													("11111111" & data_internal(15 downto 8));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3)) +
													("0000" & data_internal(15 downto 4)) +
													("00000" & data_internal(15 downto 5)) +
													("000000" & data_internal(15 downto 6)) +
													("0000000" & data_internal(15 downto 7)) +
													("00000000" & data_internal(15 downto 8));
							end if;
						when X"1" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "1111111" & ram_data_out(15 downto 7);
							else
								mix_ram_data_out <= "0000000" & ram_data_out(15 downto 7);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3)) +
													("1111" & data_internal(15 downto 4)) +
													("11111" & data_internal(15 downto 5)) +
													("111111" & data_internal(15 downto 6)) +
													("1111111" & data_internal(15 downto 7));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3)) +
													("0000" & data_internal(15 downto 4)) +
													("00000" & data_internal(15 downto 5)) +
													("000000" & data_internal(15 downto 6)) +
													("0000000" & data_internal(15 downto 7));
							end if;
						when X"2" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "111111" & ram_data_out(15 downto 6);
							else
								mix_ram_data_out <= "000000" & ram_data_out(15 downto 6);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3)) +
													("1111" & data_internal(15 downto 4)) +
													("11111" & data_internal(15 downto 5)) +
													("111111" & data_internal(15 downto 6));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3)) +
													("0000" & data_internal(15 downto 4)) +
													("00000" & data_internal(15 downto 5)) +
													("000000" & data_internal(15 downto 6));
							end if;
						when X"3" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "11111" & ram_data_out(15 downto 5);
							else
								mix_ram_data_out <= "00000" & ram_data_out(15 downto 5);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3)) +
													("1111" & data_internal(15 downto 4)) +
													("11111" & data_internal(15 downto 5));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3)) +
													("0000" & data_internal(15 downto 4)) +
													("00000" & data_internal(15 downto 5));
							end if;
						when X"4" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "1111" & ram_data_out(15 downto 4);
							else
								mix_ram_data_out <= "0000" & ram_data_out(15 downto 4);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3)) +
													("1111" & data_internal(15 downto 4));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3)) +
													("0000" & data_internal(15 downto 4));
							end if;
						when X"5" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "111" & ram_data_out(15 downto 3);
							else
								mix_ram_data_out <= "000" & ram_data_out(15 downto 3);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2)) +
													("111" & data_internal(15 downto 3));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2)) +
													("000" & data_internal(15 downto 3));
							end if;
						when X"6" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "11" & ram_data_out(15 downto 2);
							else
								mix_ram_data_out <= "00" & ram_data_out(15 downto 2);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1)) +
													("11" & data_internal(15 downto 2));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1)) +
													("00" & data_internal(15 downto 2));
							end if;
						when X"7" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= "1" & ram_data_out(15 downto 1);
							else
								mix_ram_data_out <= "0" & ram_data_out(15 downto 1);
							end if;
							if(data_internal(15) = '1') then
								mix_data_internal <=("1" & data_internal(15 downto 1));
							else
								mix_data_internal <=("0" & data_internal(15 downto 1));
							end if;
						when X"8" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("11" & data_internal(15 downto 2));
							else
								mix_data_internal <=("00" & data_internal(15 downto 2));
							end if;
						when X"9" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("111" & data_internal(15 downto 3));
							else
								mix_data_internal <=("000" & data_internal(15 downto 3));
							end if;						
						when X"A" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("1111" & data_internal(15 downto 4));
							else
								mix_data_internal <=("0000" & data_internal(15 downto 4));
							end if;						
						when X"B" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4)) +
													("11111" & ram_data_out(15 downto 5));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4)) +
													("00000" & ram_data_out(15 downto 5));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("11111" & data_internal(15 downto 5));
							else
								mix_data_internal <=("00000" & data_internal(15 downto 5));
							end if;												
						when X"C" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4)) +
													("11111" & ram_data_out(15 downto 5)) +
													("111111" & ram_data_out(15 downto 6));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4)) +
													("00000" & ram_data_out(15 downto 5)) +
													("000000" & ram_data_out(15 downto 6));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("111111" & data_internal(15 downto 6));
							else
								mix_data_internal <=("000000" & data_internal(15 downto 6));
							end if;												
						when X"D" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4)) +
													("11111" & ram_data_out(15 downto 5)) +
													("111111" & ram_data_out(15 downto 6)) +
													("1111111" & ram_data_out(15 downto 7));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4)) +
													("00000" & ram_data_out(15 downto 5)) +
													("000000" & ram_data_out(15 downto 6)) +
													("0000000" & ram_data_out(15 downto 7));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("1111111" & data_internal(15 downto 7));
							else
								mix_data_internal <=("0000000" & data_internal(15 downto 7));
							end if;												
						when X"E" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4)) +
													("11111" & ram_data_out(15 downto 5)) +
													("111111" & ram_data_out(15 downto 6)) +
													("1111111" & ram_data_out(15 downto 7)) +
													("11111111" & ram_data_out(15 downto 8));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4)) +
													("00000" & ram_data_out(15 downto 5)) +
													("000000" & ram_data_out(15 downto 6)) +
													("0000000" & ram_data_out(15 downto 7)) +
													("00000000" & ram_data_out(15 downto 8));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("11111111" & data_internal(15 downto 8));
							else
								mix_data_internal <=("00000000" & data_internal(15 downto 8));
							end if;
						when X"F" =>
							if(ram_data_out(15) = '1') then
								mix_ram_data_out <= ("1" & ram_data_out(15 downto 1)) +
													("11" & ram_data_out(15 downto 2)) +
													("111" & ram_data_out(15 downto 3)) +
													("1111" & ram_data_out(15 downto 4)) +
													("11111" & ram_data_out(15 downto 5)) +
													("111111" & ram_data_out(15 downto 6)) +
													("1111111" & ram_data_out(15 downto 7)) +
													("11111111" & ram_data_out(15 downto 8)) +
													("111111111" & ram_data_out(15 downto 9));
							else
								mix_ram_data_out <= ("0" & ram_data_out(15 downto 1)) +
													("00" & ram_data_out(15 downto 2)) +
													("000" & ram_data_out(15 downto 3)) +
													("0000" & ram_data_out(15 downto 4)) +
													("00000" & ram_data_out(15 downto 5)) +
													("000000" & ram_data_out(15 downto 6)) +
													("0000000" & ram_data_out(15 downto 7)) +
													("00000000" & ram_data_out(15 downto 8)) +
													("000000000" & ram_data_out(15 downto 9));
							end if;							
							if(data_internal(15) = '1') then
								mix_data_internal <=("111111111" & data_internal(15 downto 9));
							else
								mix_data_internal <=("000000000" & data_internal(15 downto 9));
							end if;
						end case;
						state <= X"7";
					when X"7" =>
						data_ready <= '1';
						data_out <= mix_ram_data_out + mix_data_internal;
						state <= X"0";
					when others =>
					end case;
			elsif(data_enable = '1') then
				state <= X"0";
				data_ready <= '1';
	         	data_out <= data_in;
			else
				data_ready <= '0';
				state <= X"0";
			end if;
        end if;
     end process;
end rtl;